@@ -35,7 +35,8 @@ void label_name(doscp_t *cp, const char *filename, int verbose,
 	int have_lower, have_upper;
 	wchar_t wbuffer[12];
 
-	strcpy(ans->base,"           ");
+	memset(ans, ' ', sizeof(*ans)-1);
+	ans->sentinel = '\0';
 	len = native_to_wchar(filename, wbuffer, 11, 0, 0);
 	if(len > 11){
 		*mangled = 1;
@@ -108,7 +109,7 @@ void mlabel(int argc, char **argv, int type)
 	int need_write_boot = 0;
 	int have_boot = 0;
 	char *eptr;
-	struct bootsector boot;
+	union bootsector boot;
 	Stream_t *Fs=0;
 	int r;
 	struct label_blk_t *labelBlock;
@@ -146,7 +147,7 @@ void mlabel(int argc, char **argv, int type)
 				break;
 			case 'N':
 				set_serial = SER_SET;
-				serial = strtol(optarg, &eptr, 16);
+				serial = strtoul(optarg, &eptr, 16);
 				if(*eptr) {
 					fprintf(stderr,
 						"%s not a valid serial number\n",
@@ -177,6 +178,12 @@ void mlabel(int argc, char **argv, int type)
 	if(!clear && !newLabel[0]) {
 		isRop = &isRo;
 	}
+	if(clear && newLabel[0]) {
+		/* Clear and new label specified both */
+		fprintf(stderr, "Both clear and new label specified\n");
+		FREE(&RootDir);
+		exit(1);
+	}		
 	RootDir = open_root_dir(argv[optind][0], isRop ? 0 : O_RDWR, isRop);
 	if(isRo) {
 		show = 1;
@@ -210,11 +217,20 @@ void mlabel(int argc, char **argv, int type)
 	if(interactive){
 		newLabel = longname;
 		fprintf(stderr,"Enter the new volume label : ");
-		fgets(newLabel, VBUFSIZE, stdin);
+		if(fgets(newLabel, VBUFSIZE, stdin) == NULL) {
+			newLabel[0] = '\0';
+			fprintf(stderr, "\n");
+		}
 		if(newLabel[0])
 			newLabel[strlen(newLabel)-1] = '\0';
 	}
 
+	if(strlen(newLabel) > 11) {
+		fprintf(stderr,"New label too long\n");
+		FREE(&RootDir);
+		exit(1);
+	}
+
 	if((!show || newLabel[0]) && !isNotFound(&entry)){
 		/* if we have a label, wipe it out before putting new one */
 		if(interactive && newLabel[0] == '\0')
@@ -235,14 +251,14 @@ void mlabel(int argc, char **argv, int type)
 	have_boot = 0;
 	if( (!show || newLabel[0]) || set_serial != SER_NONE) {
 		Fs = GetFs(RootDir);
-		have_boot = (force_read(Fs,(char *)&boot,0,sizeof(boot)) ==
+		have_boot = (force_read(Fs,boot.characters,0,sizeof(boot)) ==
 			     sizeof(boot));
 	}
 
-	if(_WORD(boot.fatlen)) {
-	    labelBlock = &boot.ext.old.labelBlock;
+	if(WORD_S(fatlen)) {
+	    labelBlock = &boot.boot.ext.old.labelBlock;
 	} else {
-	    labelBlock = &boot.ext.fat32.labelBlock;
+	    labelBlock = &boot.boot.ext.fat32.labelBlock;
 	}
 
 	if(!show || newLabel[0]){
@@ -256,7 +272,7 @@ void mlabel(int argc, char **argv, int type)
 		cp = GET_DOSCONVERT(Fs);
 		label_name(cp, shrtLabel, verbose, &mangled, &dosname);
 
-		if(have_boot && boot.descr >= 0xf0 &&
+		if(have_boot && boot.boot.descr >= 0xf0 &&
 		   labelBlock->dos4 == 0x29) {
 			strncpy(labelBlock->label, dosname.base, 11);
 			need_write_boot = 1;
@@ -265,7 +281,7 @@ void mlabel(int argc, char **argv, int type)
 	}
 
 	if((set_serial != SER_NONE) & have_boot) {
-		if(have_boot && boot.descr >= 0xf0 &&
+		if(have_boot && boot.boot.descr >= 0xf0 &&
 		   labelBlock->dos4 == 0x29) {
 			set_dword(labelBlock->serial, serial);	
 			need_write_boot = 1;
@@ -274,6 +290,13 @@ void mlabel(int argc, char **argv, int type)
 
 	if(need_write_boot) {
 		force_write(Fs, (char *)&boot, 0, sizeof(boot));
+		/* If this is fat 32, write backup boot sector too */
+		if(!WORD_S(fatlen)) {
+			int backupBoot = WORD_S(ext.fat32.backupBoot);
+			force_write(Fs, (char *)&boot, 
+				    backupBoot * WORD_S(secsiz),
+				    sizeof(boot));
+		}
 	}
 
 	FREE(&RootDir);
