@@ -1,4 +1,19 @@
-/*
+/*  Copyright 1995-2007,2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * Io to a plain file or device
  *
  * written by:
@@ -153,7 +168,7 @@ static int file_free(Stream_t *Stream)
 
 static int file_geom(Stream_t *Stream, struct device *dev, 
 		     struct device *orig_dev,
-		     int media, struct bootsector *boot)
+		     int media, union bootsector *boot)
 {
 	int ret;
 	DeclareThis(SimpleFile_t);
@@ -198,20 +213,20 @@ static int file_geom(Stream_t *Stream, struct device *dev,
 		InfTm = WORD(ext.old.InfTm);
 		
 		if(WORD(fatlen)) {
-			labelBlock = &boot->ext.old.labelBlock;
+			labelBlock = &boot->boot.ext.old.labelBlock;
 		} else {
-			labelBlock = &boot->ext.fat32.labelBlock;
+			labelBlock = &boot->boot.ext.fat32.labelBlock;
 		}
 
-		if (boot->descr >= 0xf0 &&
+		if (boot->boot.descr >= 0xf0 &&
 		    labelBlock->dos4 == 0x29 &&
-		    strncmp( boot->banner,"2M", 2 ) == 0 &&
+		    strncmp( boot->boot.banner,"2M", 2 ) == 0 &&
 		    BootP < 512 && Infp0 < 512 && InfpX < 512 && InfTm < 512 &&
 		    BootP >= InfTm + 2 && InfTm >= InfpX && InfpX >= Infp0 && 
 		    Infp0 >= 76 ){
 			for (sum=0, j=63; j < BootP; j++) 
-				sum += boot->jump[j];/* checksum */
-			dev->ssize = boot->jump[InfTm];
+				sum += boot->bytes[j];/* checksum */
+			dev->ssize = boot->bytes[InfTm];
 			if (!sum && dev->ssize <= 7){
 				dev->use_2m = 0xff;
 				dev->ssize |= 0x80; /* is set */
@@ -470,6 +485,7 @@ APIRET rc;
 		printOom();
 		return 0;
 	}
+	memset((void*)This, 0, sizeof(SimpleFile_t));
 	This->scsi_sector_size = 512;
 	This->seekable = 1;
 #ifdef OS_hpux
@@ -570,7 +586,11 @@ APIRET rc;
 #ifdef __EMX__
 	if (*(name+1) != ':')
 #endif
-	if (MT_FSTAT(This->fd, &This->statbuf) < 0){
+	if (MT_FSTAT(This->fd, &This->statbuf) < 0
+#ifdef OS_mingw32msvc
+	    && strncmp(name, "\\\\.\\", 4) != 0
+#endif
+	   ) {
 		Free(This);
 		if(errmsg) {
 #ifdef HAVE_SNPRINTF
